unit UService;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Controls, Vcl.SvcMgr, Vcl.Dialogs,
  IdContext, IdCustomHTTPServer, IdBaseComponent, IdComponent,
  IdCustomTCPServer, IdHTTPServer, JSON, IdServerIOHandler, IdSSL, IdSSLOpenSSL;

type
  TJSONService = class(TService)
    IdHTTPServer1: TIdHTTPServer;
    IdServerIOHandlerSSLOpenSSL1: TIdServerIOHandlerSSLOpenSSL;
    procedure IdHTTPServer1CommandGet(AContext: TIdContext;
      ARequestInfo: TIdHTTPRequestInfo; AResponseInfo: TIdHTTPResponseInfo);
    procedure ServiceStop(Sender: TService; var Stopped: Boolean);
    procedure ServicePause(Sender: TService; var Paused: Boolean);
    procedure ServiceStart(Sender: TService; var Started: Boolean);
    procedure ServiceContinue(Sender: TService; var Continued: Boolean);
    procedure IdHTTPServer1QuerySSLPort(APort: Word; var VUseSSL: Boolean);
    function IdServerIOHandlerSSLOpenSSL1VerifyPeer(Certificate: TIdX509;
      AOk: Boolean; ADepth, AError: Integer): Boolean;
  private
    { Private declarations }
    FDefaultPageNr: Integer;
    FDefaultItemCount: Integer;
  public
    function GetServiceController: TServiceController; override;
    { Public declarations }
  end;

var
  JSONService: TJSONService;

const
  LastNames: array[0..19] of string = ('Spears', 'Stark', 'Booth', 'Hendricks', 'Lutz', 'King', 'Shepard', 'Lopez', 'Harrell', 'Bolton', 'Leblanc', 'Hodges', 'Kelley',
                                        'Compton', 'Caldwell', 'Contreras', 'Tate', 'Carter', 'Richmond', 'Zimmerman');
  FirstNames: array[0..19] of string = ('Krystal', 'Samara', 'Aleah', 'Tara', 'Brianna', 'Alyssa', 'Luna', 'Marissa', 'Britney', 'Estrella',
                                       'Corey', 'Phillip', 'Brady', 'Eliezer', 'River', 'Riley', 'Jaydon', 'Matthew', 'Jairo', 'Jaydin');
  Statuses: array[0..3] of string = ('Done', 'Low priority', 'Medium priority', 'Urgent');
  Messages: array[0..39] of string = ('Some office supplies are running low.',
                                      'PC has a faulty part. New part needed.',
                                      'Release new beta', 'Release new version', 'Update depencies to newer versions',
                                      'Handle bug report #4880', 'Handle bug report #4897', 'Handle bug report #4875', 'Handle bug report #4865', 'Handle bug report #4775',
                                      'Handle bug report #4911', 'Handle bug report #4596', 'Handle bug report #4823', 'Handle bug report #4843', 'Handle bug report #4787',
                                      'Update documentation for #4153 bug fix', 'Update documentation for #4262 bug fix', 'Update documentation for #4135 bug fix', 'Update documentation for #4179 bug fix', 'Update documentation for #4199 bug fix',
                                      'Update documentation for #4177 bug fix', 'Update documentation for #4269 bug fix', 'Update documentation for #4121 bug fix', 'Update documentation for #4100 bug fix', 'Update documentation for #4148 bug fix',
                                      'Release fix for #4138', 'Release fix for #4122', 'Release fix for #4197', 'Release fix for #4148', 'Release fix for #4157',
                                      'Release fix for #4110', 'Release fix for #4186', 'Release fix for #4164', 'Release fix for #4113', 'Release fix for #4142',
                                      'Meeting for my team will start 10 minutes later', 'Meeting for my team will start 5 minutes later', 'Meeting for my team will start 15 minutes later',
                                      'Meeting for my team will start 20 minutes later', 'Meeting for my team will start 30 minutes later');

implementation

uses
  IOUtils, StrUtils;

{$R *.dfm}

procedure ServiceController(CtrlCode: DWord); stdcall;
begin
  JSONService.Controller(CtrlCode);
end;

function TJSONService.GetServiceController: TServiceController;
begin
  Result := ServiceController;
end;

procedure TJSONService.IdHTTPServer1CommandGet(AContext: TIdContext;
  ARequestInfo: TIdHTTPRequestInfo; AResponseInfo: TIdHTTPResponseInfo);
var
  paramFound: Boolean;
  pCount, ppCount, I: Integer;
  s: string;
  JSON: TJSONArray;
  obj: TJSONObject;
begin
  paramFound := False;
  pCount := -1;
  ppCount := -1;
  for I := 0 to ARequestInfo.Params.Count - 1 do
  begin
    if ARequestInfo.Params[I].StartsWith('page=') then
    begin
      s := StringReplace(ARequestInfo.Params[I], 'page=', '', [rfReplaceAll, rfIgnoreCase]);
      if TryStrToInt(s, pCount) then
        paramFound := True;
    end;

    if ARequestInfo.Params[I].StartsWith('per_page=') then
    begin
      s := StringReplace(ARequestInfo.Params[I], 'per_page=', '', [rfReplaceAll, rfIgnoreCase]);
      if TryStrToInt(s, ppCount) then
        paramFound := True;
    end;
  end;

  if paramFound then
  begin
    if (pCount <= 0) then
      pCount := FDefaultPageNr;

    if (ppCount < 0) then
      ppCount := FDefaultItemCount;

    JSON := TJSONArray.Create;
    try
      for I := (pCount - 1) * ppCount + 1 to pCount * ppCount do
      begin
        obj := TJSONObject.Create;
        obj.AddPair('first_name', FirstNames[Random(20)]);
        obj.AddPair('last_name', LastNames[Random(20)]);
        s := Statuses[Random(4)];
        case IndexStr(s, Statuses) of
          0: obj.AddPair('status_icon', 'done.png');
          1: obj.AddPair('status_icon', 'low.png');
          2: obj.AddPair('status_icon', 'medium.png');
          3: obj.AddPair('status_icon', 'urgent.png');
        end;
        obj.AddPair('status', s);
        obj.AddPair('message', Messages[Random(40)]);
        obj.AddPair('message_number', IntToStr(I));
        obj.AddPair('favs', IntToStr(Random(10)));
        JSON.Add(obj);
      end;
      AResponseInfo.ContentText := JSON.ToString;
    finally
      JSON.Free;
    end;
  end
  else
    AResponseInfo.ContentText := '[]';

  AResponseInfo.ResponseNo := 200;
  AResponseInfo.CustomHeaders.Add('Access-Control-Allow-Origin: *');
  AResponseInfo.CustomHeaders.Add('Content-Type: application/json; charset=utf-8');
  AResponseInfo.WriteContent;
end;

procedure TJSONService.IdHTTPServer1QuerySSLPort(APort: Word;
  var VUseSSL: Boolean);
begin
  VUseSSL := True;
end;

function TJSONService.IdServerIOHandlerSSLOpenSSL1VerifyPeer(
  Certificate: TIdX509; AOk: Boolean; ADepth, AError: Integer): Boolean;
begin
  if ADepth = 0 then
  begin
    Result := AOk;
  end
  else
  begin
    Result := True;
  end;
end;

procedure TJSONService.ServiceContinue(Sender: TService;
  var Continued: Boolean);
begin
  IdHTTPServer1.Active := True;
  Continued := True;
end;

procedure TJSONService.ServicePause(Sender: TService; var Paused: Boolean);
begin
  IdHTTPServer1.Active := False;
  Paused := True;
end;

procedure TJSONService.ServiceStart(Sender: TService; var Started: Boolean);
begin
  IdHTTPServer1.Active := True;
  Started := True;
end;

procedure TJSONService.ServiceStop(Sender: TService; var Stopped: Boolean);
begin
  IdHTTPServer1.Active := False;
  Stopped := True;
end;

end.
